package cn.com.qmth.scancentral.controller.admin;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.ExamService;
import cn.com.qmth.scancentral.service.StudentImportService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.util.ResouceUtil;
import cn.com.qmth.scancentral.vo.ResultVo;
import cn.com.qmth.scancentral.vo.studentimport.StudentCountVo;
import cn.com.qmth.scancentral.vo.studentimport.StudentImportConfigVo;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.uuid.FastUUID;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Api(tags = "考生导入相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/student")
@Aac(strict = false, auth = true)
public class StudentImportController extends BaseController {

    @Autowired
    private StudentService studentService;

    @Autowired
    private StudentImportService studentImportService;

    @Autowired
    private ExamService examService;

    @ApiOperation(value = "考生导入")
    @PostMapping("/import")
    public Map<String, Object> studentImport(@RequestParam Long examId, @RequestParam MultipartFile file) throws Exception {
        if (studentImportService.existRunningStudentImportTask()) {
            throw new StatusException("有正在运行的考生导入任务，请稍后再试！");
        }

        final String taskId = FastUUID.get();
        studentImportService.studentImport(taskId, examId, file.getOriginalFilename(), file.getBytes());

        Map<String, Object> result = new HashMap<>();
        result.put("taskId", taskId);
        return result;
    }

    @ApiOperation(value = "考生导入进度查询")
    @PostMapping("/import/progress")
    public Map<String, Object> studentImportProgress(@RequestParam String taskId) {
        return studentImportService.studentImportProgress(taskId);
    }

    @ApiOperation(value = "考生导入报告下载", hidden = true)
    @GetMapping("/import/report/{taskId}")
    public void importReport(@PathVariable String taskId) {
        File logFile = studentImportService.studentImportReport(taskId);
        super.exportFile("考生导入报告.txt", logFile);
    }

    @ApiOperation(value = "查询科目考生数")
    @PostMapping("/count")
    public List<StudentCountVo> countStudent(@RequestParam Long examId) {
        return studentService.countStudent(examId);
    }

    @ApiOperation(value = "考生清空")
    @PostMapping(value = "/clear")
    public ResultVo studentClean(@RequestParam Long examId, @RequestParam String subjectCode) {
        studentService.studentClean(examId, subjectCode);
        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "获取考生导入参数设置")
    @PostMapping(value = "/import/config")
    public StudentImportConfigVo getStudentImportConfig(@RequestParam Long examId) {
        return examService.getStudentImportConfig(examId);
    }

    @ApiOperation(value = "保存考生导入参数设置")
    @PostMapping(value = "/import/config/save")
    public ResultVo saveStudentImportConfig(StudentImportConfigVo config) {
        examService.saveStudentImportConfig(config);
        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "考生导入模版下载")
    @GetMapping("/import/template")
    public void importTemplate() {
        super.exportFile("考生导入模板.txt", ResouceUtil.getStream("templates/student-import.txt"));
    }

}
